@@ -93,6 +93,7 @@ public:
     virtual void begin(int shortcut, QEvent *event);
     /**
      * End the action.
+     *
      * \param event The mouse event that has finished this action.
      *              Is null for keyboard-activated actions.
      */
@@ -167,8 +168,8 @@ public:
     virtual bool isShortcutRequired(int shortcut) const;
 
     /**
-     * Some of the actions are available in particular sutiations
-     * only.  E.g. switch frame action is available iff a animated
+     * Some of the actions are available in particular situations
+     * only.  E.g. switch frame action is available iff an animated
      * layer is selected. If isAvailable() returns true then the
      * action will *not* be triggered by the shortcut matcher.
      */
@@ -203,6 +204,7 @@ protected:
      * The default implementation of inputEvent calls this function.
      */
     virtual void cursorMoved(const QPointF &lastPos, const QPointF &pos);
+    virtual void cursorMovedAbsolute(const QPointF &startPos, const QPointF &pos);
 
     /**
      * Convenience method to extract the position from a cursor movement event.
