@@ -38,10 +38,6 @@
 
 struct KisResourcesSnapshot::Private {
     Private()
-        : currentPattern(0)
-        , currentGradient(0)
-        , currentGenerator(0)
-        , compositeOp(0)
     {
     }
 
@@ -49,31 +45,33 @@ struct KisResourcesSnapshot::Private {
     KisDefaultBoundsBaseSP bounds;
     KoColor currentFgColor;
     KoColor currentBgColor;
-    KoPattern *currentPattern = 0;
-    KoAbstractGradient *currentGradient;
+    KoPattern *currentPattern {0};
+    KoAbstractGradient *currentGradient {0};
     KisPaintOpPresetSP currentPaintOpPreset;
-    KisNodeSP currentNode;
-    qreal currentExposure;
+    KisNodeSP currentNode {0};
+    qreal currentExposure {0.0};
     KisFilterConfigurationSP currentGenerator;
 
     QPointF axesCenter;
-    bool mirrorMaskHorizontal = false;
-    bool mirrorMaskVertical = false;
+    bool mirrorMaskHorizontal {false};
+    bool mirrorMaskVertical {false};
 
-    quint8 opacity = OPACITY_OPAQUE_U8;
-    QString compositeOpId = COMPOSITE_OVER;
-    const KoCompositeOp *compositeOp;
+    quint8 opacity {OPACITY_OPAQUE_U8};
+    QString compositeOpId {COMPOSITE_OVER};
+    const KoCompositeOp *compositeOp {0};
 
-    KisPainter::StrokeStyle strokeStyle = KisPainter::StrokeStyleBrush;
-    KisPainter::FillStyle fillStyle = KisPainter::FillStyleForegroundColor;
+    KisPainter::StrokeStyle strokeStyle {KisPainter::StrokeStyleBrush};
+    KisPainter::FillStyle fillStyle {KisPainter::FillStyleForegroundColor};
+    QTransform fillTransform;
 
-    bool globalAlphaLock = false;
-    qreal effectiveZoom = 1.0;
-    bool presetAllowsLod = false;
+    bool globalAlphaLock {false};
+    qreal effectiveZoom {1.0};
+    bool presetAllowsLod {false};
     KisSelectionSP selectionOverride;
+    bool hasOverrideSelection {false};
 };
 
-KisResourcesSnapshot::KisResourcesSnapshot(KisImageSP image, KisNodeSP currentNode, KoCanvasResourceManager *resourceManager, KisDefaultBoundsBaseSP bounds)
+KisResourcesSnapshot::KisResourcesSnapshot(KisImageSP image, KisNodeSP currentNode, KoCanvasResourceProvider *resourceManager, KisDefaultBoundsBaseSP bounds)
     : m_d(new Private())
 {
     m_d->image = image;
@@ -81,18 +79,21 @@ KisResourcesSnapshot::KisResourcesSnapshot(KisImageSP image, KisNodeSP currentNo
         bounds = new KisDefaultBounds(m_d->image);
     }
     m_d->bounds = bounds;
-    m_d->currentFgColor = resourceManager->resource(KoCanvasResourceManager::ForegroundColor).value<KoColor>();
-    m_d->currentBgColor = resourceManager->resource(KoCanvasResourceManager::BackgroundColor).value<KoColor>();
+    m_d->currentFgColor = resourceManager->resource(KoCanvasResourceProvider::ForegroundColor).value<KoColor>();
+    m_d->currentBgColor = resourceManager->resource(KoCanvasResourceProvider::BackgroundColor).value<KoColor>();
     m_d->currentPattern = resourceManager->resource(KisCanvasResourceProvider::CurrentPattern).value<KoPattern*>();
     m_d->currentGradient = resourceManager->resource(KisCanvasResourceProvider::CurrentGradient).value<KoAbstractGradient*>();
 
     /**
-     * We should deep-copy the preset, so that long-runnign actions
-     * will have correct brush parameters. Theoretically this cloniong
+     * We should deep-copy the preset, so that long-running actions
+     * will have correct brush parameters. Theoretically this cloning
      * can be expensive, but according to measurements, it takes
      * something like 0.1 ms for an average preset.
      */
-    m_d->currentPaintOpPreset = resourceManager->resource(KisCanvasResourceProvider::CurrentPaintOpPreset).value<KisPaintOpPresetSP>()->clone();
+    KisPaintOpPresetSP p = resourceManager->resource(KisCanvasResourceProvider::CurrentPaintOpPreset).value<KisPaintOpPresetSP>();
+    if (p) {
+        m_d->currentPaintOpPreset = resourceManager->resource(KisCanvasResourceProvider::CurrentPaintOpPreset).value<KisPaintOpPresetSP>()->clone();
+    }
 
 #ifdef HAVE_THREADED_TEXT_RENDERING_WORKAROUND
     KisPaintOpRegistry::instance()->preinitializePaintOpIfNeeded(m_d->currentPaintOpPreset);
@@ -106,7 +107,7 @@ KisResourcesSnapshot::KisResourcesSnapshot(KisImageSP image, KisNodeSP currentNo
     if (m_d->image) {
         relativeAxesCenter = m_d->image->mirrorAxesCenter();
     }
-    m_d->axesCenter = KisAlgebra2D::relativeToAbsolute(relativeAxesCenter, m_d->bounds->bounds());
+    m_d->axesCenter = KisAlgebra2D::relativeToAbsolute(relativeAxesCenter, m_d->bounds->imageBorderRect());
 
     m_d->mirrorMaskHorizontal = resourceManager->resource(KisCanvasResourceProvider::MirrorHorizontal).toBool();
     m_d->mirrorMaskVertical = resourceManager->resource(KisCanvasResourceProvider::MirrorVertical).toBool();
@@ -150,7 +151,7 @@ KisResourcesSnapshot::KisResourcesSnapshot(KisImageSP image, KisNodeSP currentNo
     if (m_d->image) {
         relativeAxesCenter = m_d->image->mirrorAxesCenter();
     }
-    m_d->axesCenter = KisAlgebra2D::relativeToAbsolute(relativeAxesCenter, m_d->bounds->bounds());
+    m_d->axesCenter = KisAlgebra2D::relativeToAbsolute(relativeAxesCenter, m_d->bounds->imageBorderRect());
     m_d->opacity = OPACITY_OPAQUE_U8;
 
     setCurrentNode(currentNode);
@@ -190,6 +191,8 @@ void KisResourcesSnapshot::setupPainter(KisPainter* painter)
 
     painter->setStrokeStyle(m_d->strokeStyle);
     painter->setFillStyle(m_d->fillStyle);
+    painter->setPatternTransform(m_d->fillTransform);
+
 
     /**
      * The paintOp should be initialized the last, because it may
@@ -214,6 +217,8 @@ void KisResourcesSnapshot::setupMaskingBrushPainter(KisPainter *painter)
 
     painter->setMirrorInformation(m_d->axesCenter, m_d->mirrorMaskHorizontal, m_d->mirrorMaskVertical);
 
+    painter->setStrokeStyle(m_d->strokeStyle);
+
     /**
      * The paintOp should be initialized the last, because it may
      * ask the painter for some options while initialization
@@ -250,6 +255,11 @@ void KisResourcesSnapshot::setFillStyle(KisPainter::FillStyle fillStyle)
     m_d->fillStyle = fillStyle;
 }
 
+void KisResourcesSnapshot::setFillTransform(QTransform transform)
+{
+    m_d->fillTransform = transform;
+}
+
 KisNodeSP KisResourcesSnapshot::currentNode() const
 {
     return m_d->currentNode;
@@ -267,7 +277,9 @@ bool KisResourcesSnapshot::needsIndirectPainting() const
 
 QString KisResourcesSnapshot::indirectPaintingCompositeOp() const
 {
-    return m_d->currentPaintOpPreset->settings()->indirectPaintingCompositeOp();
+    return m_d->currentPaintOpPreset ?
+            m_d->currentPaintOpPreset->settings()->indirectPaintingCompositeOp()
+              : COMPOSITE_ALPHA_DARKEN;
 }
 
 bool KisResourcesSnapshot::needsMaskingBrushRendering() const
@@ -281,7 +293,7 @@ KisSelectionSP KisResourcesSnapshot::activeSelection() const
      * It is possible to have/use the snapshot without the image. Such
      * usecase is present for example in the scratchpad.
      */
-    if (m_d->selectionOverride) {
+    if (m_d->hasOverrideSelection) {
         return m_d->selectionOverride;
     }
 
@@ -302,17 +314,23 @@ KisSelectionSP KisResourcesSnapshot::activeSelection() const
 
 bool KisResourcesSnapshot::needsAirbrushing() const
 {
-    return m_d->currentPaintOpPreset->settings()->isAirbrushing();
+    return (   m_d->currentPaintOpPreset
+            && m_d->currentPaintOpPreset->settings()
+            && m_d->currentPaintOpPreset->settings()->isAirbrushing());
 }
 
 qreal KisResourcesSnapshot::airbrushingInterval() const
 {
-    return m_d->currentPaintOpPreset->settings()->airbrushInterval();
+    return (   m_d->currentPaintOpPreset
+            && m_d->currentPaintOpPreset->settings()
+            && m_d->currentPaintOpPreset->settings()->airbrushInterval());
 }
 
 bool KisResourcesSnapshot::needsSpacingUpdates() const
 {
-    return m_d->currentPaintOpPreset->settings()->useSpacingUpdates();
+    return (   m_d->currentPaintOpPreset
+            && m_d->currentPaintOpPreset->settings()
+            && m_d->currentPaintOpPreset->settings()->useSpacingUpdates());
 }
 
 void KisResourcesSnapshot::setOpacity(qreal opacity)
@@ -355,6 +373,15 @@ KisPaintOpPresetSP KisResourcesSnapshot::currentPaintOpPreset() const
     return m_d->currentPaintOpPreset;
 }
 
+KoAbstractGradient* KisResourcesSnapshot::currentGradient() const
+{
+    return m_d->currentGradient;
+}
+
+QTransform KisResourcesSnapshot::fillTransform() const
+{
+    return m_d->fillTransform;
+}
 
 QBitArray KisResourcesSnapshot::channelLockFlags() const
 {
@@ -402,6 +429,7 @@ void KisResourcesSnapshot::setBGColorOverride(const KoColor &color)
 void KisResourcesSnapshot::setSelectionOverride(KisSelectionSP selection)
 {
     m_d->selectionOverride = selection;
+    m_d->hasOverrideSelection = true; // needed if selection passed is null to ignore selection
 }
 
 void KisResourcesSnapshot::setBrush(const KisPaintOpPresetSP &brush)
