@@ -39,6 +39,7 @@
 #include "KoIntegerMaths.h"
 #include <KoUpdater.h>
 
+#include <filter/kis_filter_category_ids.h>
 #include <filter/kis_filter_registry.h>
 #include <filter/kis_filter.h>
 #include <kis_global.h>
@@ -53,11 +54,11 @@
 
 
 KisRainDropsFilter::KisRainDropsFilter()
-    : KisFilter(id(), KisFilter::categoryArtistic(), i18n("&Raindrops..."))
+    : KisFilter(id(), FiltersCategoryArtisticId, i18n("&Raindrops..."))
 {
     setSupportsPainting(false);
     setSupportsThreading(false);
-    setSupportsAdjustmentLayers(true);
+    setSupportsAdjustmentLayers(false);
 }
 
 // This method have been ported from Pieter Z. Voloshyn algorithm code.
@@ -132,7 +133,7 @@ void KisRainDropsFilter::processImpl(KisPaintDeviceSP device,
     }
 
     progressUpdater->setRange(0, number);
-    KisRandomAccessorSP dstAccessor = device->createRandomAccessorNG(srcTopLeft.x(), srcTopLeft.y());
+    KisRandomAccessorSP dstAccessor = device->createRandomAccessorNG();
     
     for (uint NumBlurs = 0; NumBlurs <= number; ++NumBlurs) {
         NewSize = (int)(qrand() * ((double)(DropSize - 5) / RAND_MAX) + 5);
@@ -294,6 +295,13 @@ void KisRainDropsFilter::processImpl(KisPaintDeviceSP device,
                     if ((m >= 0) && (m < Height) && (n >= 0) && (n < Width)) {
                         QColor color;
 
+                        if (BlurPixels == 0) {
+                            // Coverity complains that it *is* possible
+                            // for BlurPixels to be 0, so let's make sure
+                            // Krita doesn't crash here
+                            BlurPixels = 1;
+                        }
+
                         color.setRgb((int)(R / BlurPixels), (int)(G / BlurPixels), (int)(B / BlurPixels));
                         dstAccessor->moveTo(srcTopLeft.x() + n, srcTopLeft.y() + m);
                         cs->fromQColor(color, dstAccessor->rawData());
@@ -372,20 +380,20 @@ uchar KisRainDropsFilter::LimitValues(int ColorValue) const
     return ((uchar) ColorValue);
 }
 
-KisConfigWidget * KisRainDropsFilter::createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP) const
+KisConfigWidget * KisRainDropsFilter::createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP, bool) const
 {
     vKisIntegerWidgetParam param;
     param.push_back(KisIntegerWidgetParam(1, 200, 80, i18n("Drop size"), "dropsize"));
-    param.push_back(KisIntegerWidgetParam(1, 500, 80, i18n("Number"), "number"));
+    param.push_back(KisIntegerWidgetParam(1, 500, 80, i18n("Number of drops"), "number"));
     param.push_back(KisIntegerWidgetParam(1, 100, 30, i18n("Fish eyes"), "fishEyes"));
     KisMultiIntegerFilterWidget * w = new KisMultiIntegerFilterWidget(id().id(), parent, id().id(), param);
-    w->setConfiguration(factoryConfiguration());
+    w->setConfiguration(defaultConfiguration());
     return w;
 }
 
-KisFilterConfigurationSP KisRainDropsFilter::factoryConfiguration() const
+KisFilterConfigurationSP KisRainDropsFilter::defaultConfiguration() const
 {
-    KisFilterConfigurationSP config = new KisFilterConfiguration("raindrops", 2);
+    KisFilterConfigurationSP config = factoryConfiguration();
     config->setProperty("dropsize", 80);
     config->setProperty("number", 80);
     config->setProperty("fishEyes", 30);
