@@ -22,7 +22,6 @@
  */
 
 #include "kis_tool_crop.h"
-#include "kistoolcropconfigwidget.h"
 
 
 #include <QCheckBox>
@@ -33,6 +32,7 @@
 #include <QPushButton>
 #include <QRect>
 #include <QVector>
+#include <QMenu>
 
 #include <kis_debug.h>
 #include <klocalizedstring.h>
@@ -121,6 +121,15 @@ KisToolCrop::KisToolCrop(KoCanvasBase * canvas)
     connect(&m_finalRect, SIGNAL(sigValuesChanged()), SLOT(slotRectChanged()));
     connect(&m_finalRect, SIGNAL(sigLockValuesChanged()), SLOT(slotRectChanged()));
 
+    // context menu options (mirrors tool options)
+    m_contextMenu.reset(new QMenu());
+    applyCrop = new KisAction(i18n("Crop"));
+
+    centerToggleOption = new KisAction(i18n("Center"));
+    centerToggleOption->setCheckable(true);
+    
+    growToggleOption = new KisAction(i18n("Grow"));
+    growToggleOption->setCheckable(true);
 }
 
 KisToolCrop::~KisToolCrop()
@@ -209,6 +218,29 @@ void KisToolCrop::paint(QPainter &painter, const KoViewConverter &converter)
     paintOutlineWithHandles(painter);
 }
 
+QMenu *KisToolCrop::popupActionsMenu()
+{
+    if (m_contextMenu) {
+        // Sync state of context menu toggles with state of Tool Options toggles
+        centerToggleOption->setChecked(growCenter());
+        growToggleOption->setChecked(allowGrow());
+        m_contextMenu->clear();
+
+        m_contextMenu->addSection(i18n("Crop Tool Actions"));
+        m_contextMenu->addSeparator();
+
+        if (m_haveCropSelection) {         // can't crop if there is no selection
+            m_contextMenu->addAction(applyCrop);
+            m_contextMenu->addSeparator();
+        }
+
+        m_contextMenu->addAction(centerToggleOption);
+        m_contextMenu->addAction(growToggleOption);
+    }
+
+    return m_contextMenu.data();
+}
+
 void KisToolCrop::beginPrimaryAction(KoPointerEvent *event)
 {
     m_finalRect.setCropRect(image()->bounds());
@@ -485,9 +517,9 @@ void KisToolCrop::slotRectChanged()
     emit cropXChanged(cropX());
     emit cropYChanged(cropY());
     emit ratioChanged(ratio());
-    emit forceHeightChanged(forceHeight());
-    emit forceWidthChanged(forceWidth());
-    emit forceRatioChanged(forceRatio());
+    emit lockHeightChanged(lockHeight());
+    emit lockWidthChanged(lockWidth());
+    emit lockRatioChanged(lockRatio());
 
     emit canGrowChanged(allowGrow());
     emit isCenteredChanged(growCenter());
@@ -553,12 +585,12 @@ int KisToolCrop::cropWidth() const
     return m_finalRect.rect().width();
 }
 
-void KisToolCrop::setForceWidth(bool force)
+void KisToolCrop::setLockWidth(bool lock)
 {
-    m_finalRect.setWidthLocked(force);
+    m_finalRect.setWidthLocked(lock);
 }
 
-bool KisToolCrop::forceWidth() const
+bool KisToolCrop::lockWidth() const
 {
     return m_finalRect.widthLocked();
 }
@@ -581,12 +613,12 @@ int KisToolCrop::cropHeight() const
     return m_finalRect.rect().height();
 }
 
-void KisToolCrop::setForceHeight(bool force)
+void KisToolCrop::setLockHeight(bool lock)
 {
-    m_finalRect.setHeightLocked(force);
+    m_finalRect.setHeightLocked(lock);
 }
 
-bool KisToolCrop::forceHeight() const
+bool KisToolCrop::lockHeight() const
 {
     return m_finalRect.heightLocked();
 }
@@ -596,6 +628,8 @@ void KisToolCrop::setAllowGrow(bool g)
     m_finalRect.setCanGrow(g);
     m_finalRect.setCropRect(image()->bounds());
     configGroup.writeEntry("allowGrow", g);
+
+    emit canGrowChanged(g);
 }
 
 bool KisToolCrop::allowGrow() const
@@ -606,7 +640,11 @@ bool KisToolCrop::allowGrow() const
 void KisToolCrop::setGrowCenter(bool value)
 {
     m_finalRect.setCentered(value);
+
+
     configGroup.writeEntry("growCenter", value);
+
+    emit isCenteredChanged(value);
 }
 
 bool KisToolCrop::growCenter() const
@@ -632,19 +670,19 @@ double KisToolCrop::ratio() const
     return m_finalRect.ratio();
 }
 
-void KisToolCrop::setForceRatio(bool force)
+void KisToolCrop::setLockRatio(bool lock)
 {
-    m_finalRect.setRatioLocked(force);
+    m_finalRect.setRatioLocked(lock);
 }
 
-bool KisToolCrop::forceRatio() const
+bool KisToolCrop::lockRatio() const
 {
     return m_finalRect.ratioLocked();
 }
 
 QWidget* KisToolCrop::createOptionWidget()
 {
-    KisToolCropConfigWidget* optionsWidget = new KisToolCropConfigWidget(0, this);
+    optionsWidget = new KisToolCropConfigWidget(0, this);
     // See https://bugs.kde.org/show_bug.cgi?id=316896
     QWidget *specialSpacer = new QWidget(optionsWidget);
     specialSpacer->setObjectName("SpecialSpacer");
@@ -660,17 +698,21 @@ QWidget* KisToolCrop::createOptionWidget()
     connect(optionsWidget, SIGNAL(cropXChanged(int)), this, SLOT(setCropX(int)));
     connect(optionsWidget, SIGNAL(cropYChanged(int)), this, SLOT(setCropY(int)));
     connect(optionsWidget, SIGNAL(cropHeightChanged(int)), this, SLOT(setCropHeight(int)));
-    connect(optionsWidget, SIGNAL(forceHeightChanged(bool)), this, SLOT(setForceHeight(bool)));
+    connect(optionsWidget, SIGNAL(lockHeightChanged(bool)), this, SLOT(setLockHeight(bool)));
     connect(optionsWidget, SIGNAL(cropWidthChanged(int)), this, SLOT(setCropWidth(int)));
-    connect(optionsWidget, SIGNAL(forceWidthChanged(bool)), this, SLOT(setForceWidth(bool)));
+    connect(optionsWidget, SIGNAL(lockWidthChanged(bool)), this, SLOT(setLockWidth(bool)));
     connect(optionsWidget, SIGNAL(ratioChanged(double)), this, SLOT(setRatio(double)));
-    connect(optionsWidget, SIGNAL(forceRatioChanged(bool)), this, SLOT(setForceRatio(bool)));
+    connect(optionsWidget, SIGNAL(lockRatioChanged(bool)), this, SLOT(setLockRatio(bool)));
     connect(optionsWidget, SIGNAL(decorationChanged(int)), this, SLOT(setDecoration(int)));
     connect(optionsWidget, SIGNAL(allowGrowChanged(bool)), this, SLOT(setAllowGrow(bool)));
     connect(optionsWidget, SIGNAL(growCenterChanged(bool)), this, SLOT(setGrowCenter(bool)));
 
     optionsWidget->setFixedHeight(optionsWidget->sizeHint().height());
 
+    connect(applyCrop, SIGNAL(triggered(bool)), this, SLOT(crop()));
+    connect(centerToggleOption, SIGNAL(triggered(bool)), this, SLOT(setGrowCenter(bool)));
+    connect(growToggleOption, SIGNAL(triggered(bool)), this, SLOT(setAllowGrow(bool)));
+
     return optionsWidget;
 }
 
